@@ -23,52 +23,14 @@ one way (or not at all) when the decorator is originally evaluated, but then to
 be wrapped differently once the config has loaded and the desired final
 behavior has been established.
 """
-from metlog.client import MetlogClient
-from metlog.config import client_from_dict_config
+import functools
+
 from metlog.decorators.util import return_fq_name
+from metlog.holder import CLIENT_HOLDER
 try:
     import json
 except:
-    import simplejson as json
-
-
-class MetlogClientWrapper(object):
-    """
-    This class acts as a lazy proxy of sorts to the MetlogClient. We need this
-    to provide late binding of the MetlogClient so that decorators which use
-    Metlog have a chance to be configured prior to affecting the callable which
-    is being decorated.
-    """
-    def __init__(self):
-        self.reset()
-
-    def activate(self, client_config):
-        """
-        Applies configuration to the wrapped client, allowing it to be used and
-        activating any Metlog decorators that might be in use.
-
-        :param client_config: Dictionary containing MetlogClient configuration.
-        """
-        client_from_dict_config(client_config, client=self.client)
-        disabled_decorators = [k.replace("disable_", '')
-                               for (k, v) in client_config.items()
-                               if (k.startswith('disable_') and v)]
-        self._disabled_decorators = set(disabled_decorators)
-        self.is_activated = True
-
-    def reset(self):
-        """
-        Sets client related instance variables to default settings.
-        """
-        self.client = MetlogClient()
-        self._disabled_decorators = set()
-        self.is_activated = False
-
-    def decorator_is_disabled(self, name):
-        # Check if this particular logger is disabled
-        return name in self._disabled_decorators
-
-CLIENT_WRAPPER = MetlogClientWrapper()
+    import simplejson as json  # NOQA
 
 
 class MetlogDecorator(object):
@@ -82,6 +44,17 @@ class MetlogDecorator(object):
     but call the wrapped function) will be used as the decorator.
     """
     def __init__(self, *args, **kwargs):
+        """
+        :param client: Optional MetlogClient instance. Will override any
+                       `client_name` value that may be specified, if provided.
+        :param client_name: Optional `logger` name of a MetlogClient instance
+                            that is stored in the CLIENT_HOLDER
+
+        If neither the `client` nor `client_name` parameters are specified,
+        then CLIENT_HOLDER.default_client will be used.
+        """
+        self._client = kwargs.pop('client', None)
+        self.client_name = kwargs.pop('client_name', '')
         if len(args) == 1 and len(kwargs) == 0 and callable(args[0]):
             # bare decorator, i.e. no arguments
             self.args = None
@@ -98,15 +71,23 @@ class MetlogDecorator(object):
     def decorator_name(self):
         return self.__class__.__name__
 
+    @property
+    def client(self):
+        if self._client is None:
+            if self.client_name:
+                self._client = CLIENT_HOLDER.get_client(self.client_name)
+            else:
+                self._client = CLIENT_HOLDER.default_client
+        return self._client
+
     def predicate(self):
         """
         Called during the rebind process. True return value will rebind such
         that `self.metlog_call` becomes the decorator function, False will
         rebind such that `self._invoke` becomes the decorator function.
         """
-        if not CLIENT_WRAPPER.is_activated:
-            return False
-        if CLIENT_WRAPPER.decorator_is_disabled(self.decorator_name):
+        disabled = CLIENT_HOLDER.global_config.get('disabled_decorators', [])
+        if self.decorator_name in disabled:
             return False
         return True
 
@@ -142,7 +123,7 @@ class MetlogDecorator(object):
         else:
             sorted_args = tuple(self.args)
 
-        key = (self.__class__, self.args, sorted_kw)
+        key = (self.__class__, sorted_args, sorted_kw)
 
         self._metlog_decorators.add(key)
 
@@ -169,6 +150,14 @@ class MetlogDecorator(object):
         self.__call__ = replacement
         return replacement(*args, **kwargs)
 
+    def __get__(self, instance, owner):
+        """Descriptor lookup logic to implement bound methods."""
+        # If accessed directly from the class, return the decorator itself.
+        if instance is None:
+            return self
+        # If accessed via an instance, bind it as the first argument.
+        return functools.partial(self, instance)
+
     @property
     def __name__(self):
         """Support the use of functools.wraps."""
